/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "Group.h"

Group::Group(const char* nameIn, HWND parentWnd)
{
	name = (char*)malloc(strlen(nameIn) + 1);
	memcpy(name, nameIn, strlen(nameIn) + 1);
	_parentWnd = parentWnd;

	PrivateExtractIcons = (UINT (WINAPI *)(LPCTSTR,int,int,int,HICON*,UINT*,UINT,UINT)) GetProcAddress(GetModuleHandle("USER32.DLL"), "PrivateExtractIconsA");
	if(!PrivateExtractIcons) {
		DWORD dw = GetLastError();
		LSLogPrintf(LOG_ERROR, "IconDesk", "Could not bind PrivateExtractIcons (Error: %u)", dw);
	}

	_iconsLoaded = FALSE;

	char buffer[MAX_LINE_LENGTH];

	_visible = _getBool("Visible", TRUE);
	_iconX = _getInt("X", 0);
	_iconY = _getInt("Y", 0);
	if(_iconX < 0) _iconX += GetSystemMetrics(SM_CXSCREEN);
	if(_iconY < 0) _iconY += GetSystemMetrics(SM_CYSCREEN);
	_iconSize = _getInt("Size", 32);
	_iconSpacingX = _getInt("SpacingX", 0);
	_iconSpacingY = _getInt("SpacingY", 0);

	// Folders
	FILE* fpStep;
	fpStep = LCOpen(NULL);
	if(fpStep) {
		char szPrefix[MAX_LINE_LENGTH], szBuffer[MAX_LINE_LENGTH], szData[MAX_LINE_LENGTH], szDummy[MAX_LINE_LENGTH];
		char* rszTokens[1];

		rszTokens[0] = szDummy;
		sprintf(szPrefix, "*IconDesk%sFolder", name);

		szBuffer[0] = 0;
		while(LCReadNextConfig(fpStep, szPrefix, szBuffer, sizeof(szBuffer))) {
			szDummy[0] = szData[0] = 0;
			LCTokenize(szBuffer, rszTokens, 1, szData);

			// Remove quotes around the path
			if(szData[0] == '"' && szData[strlen(szData) - 1] == '"') {
				strcpy(szData, szData + 1);
				szData[strlen(szData) - 1] = '\0';
			}

			_folders.push_back(szData);

			LSLogPrintf(LOG_DEBUG, "IconDesk", "Folder \"%s\" \"%s\"", name, szData);

			// Folder watcher
			LPITEMIDLIST pidl = NULL;
			IShellFolder *psfDesktop = NULL;
			HRESULT hr;
			WCHAR wszFolder[MAX_PATH];

			if(!stricmp(szData, ".desktop")) {
				LPMALLOC pMalloc;
				HRESULT hr = SHGetMalloc(&pMalloc);
				void *temp = pMalloc->Alloc(2);
			    pMalloc->Release();
				ZeroMemory(temp, 2);
				pidl = (LPITEMIDLIST)temp;
			}
			else {
				// Get the IShellFolder for the folder
				MultiByteToWideChar(CP_ACP, 0, szData, -1, wszFolder, sizeof(wszFolder) / sizeof(WCHAR));
				// Used to bind the interface
				hr = SHGetDesktopFolder(&psfDesktop);
				// Get the PIDL
				hr = psfDesktop->ParseDisplayName(NULL, NULL, wszFolder, NULL, &pidl, NULL);
				// Release the desktop
				psfDesktop->Release();
			}
			_notifiers.push_back(new ShellNotify(pidl, this));
		}
		LCClose(fpStep);
	}

	if(_folders.size() == 0) {
		_folders.push_back("C:\\");
		// Folder watcher
		LPITEMIDLIST pidl = NULL;
		IShellFolder *psfDesktop = NULL;
		HRESULT hr;
		WCHAR wszFolder[MAX_PATH];

		// Get the IShellFolder for the folder
		MultiByteToWideChar(CP_ACP, 0, "C:\\", -1, wszFolder, sizeof(wszFolder) / sizeof(WCHAR));
		// Used to bind the interface
		hr = SHGetDesktopFolder(&psfDesktop);
		// Get the PIDL
		hr = psfDesktop->ParseDisplayName(NULL, NULL, wszFolder, NULL, &pidl, NULL);
		// Release the desktop
		psfDesktop->Release();

		_notifiers.push_back(new ShellNotify(pidl, this));
	}

	_iconHideExtension = _getBool("HideExtension", TRUE);
	_iconFoldersFirst = _getBool("FoldersFirst", TRUE);
	_iconShowHidden = _getBool("ShowHidden", FALSE);
	_showTooltips = _getBool("Tooltips", TRUE);

	_hideText = _getBool("HideText", FALSE);
	if(!_hideText) {
		_getString("TextBackground", buffer, "", MAX_LINE_LENGTH);
		if(strlen(buffer)) {
			_iconTextBackground = CreateSolidBrush(_getColor("TextBackground", RGB(255,255,255)));
		}
		else {
			_iconTextBackground = NULL;
		}
		_iconTextSpacing = _getInt("TextSpacing", 5);
		_iconTextHeight = _getInt("TextHeight", 28);
		_iconTextWidth = _getInt("TextWidth", 64);

		_iconTextColor = _getColor("FontColor", RGB(255, 255, 255));

		_getString("Font", buffer, "Arial", MAX_LINE_LENGTH);
		int fontSize = _getInt("FontSize", 14);
		BOOL fontItalic = _getBool("FontItalic", FALSE);
		BOOL fontBold = _getBool("FontBold", FALSE);
		if(!(_iconTextFont = CreateFont(fontSize, 0, 0, 0, fontBold ? FW_BOLD : FW_NORMAL, fontItalic, FALSE, FALSE, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, buffer))) {
			LSLogPrintf(LOG_ERROR, "IconDesk", "Could not load font \"%s\"", buffer);
		}

		_textShadowColor = _getColor("TextShadowColor", RGB(80, 80, 80));
		_textShadowDepth = _getInt("TextShadowDepth", 0);
	}
	else {
		_iconTextWidth = 0;
		_iconTextHeight = 0;
		_iconTextSpacing = 0;
	}

	_getString("Direction", buffer, "", MAX_LINE_LENGTH);
	if(!stricmp(buffer, "down")) {
		_iconDirection = DD_DOWN;
	}
	else if(!stricmp(buffer, "left")) {
		_iconDirection = DD_LEFT;
	}
	else if(!stricmp(buffer, "up")) {
		_iconDirection = DD_UP;
	}
	else {
		_iconDirection = DD_RIGHT;
	}

	_getString("WrapDirection", buffer, "", MAX_LINE_LENGTH);
	if(_iconDirection == DD_DOWN || _iconDirection == DD_UP) {
		if(!stricmp(buffer, "left")) {
			_iconWrapDirection = DD_LEFT;
		}
		else {
			_iconWrapDirection = DD_RIGHT;
		}
	}
	else {
		if(!stricmp(buffer, "up")) {
			_iconWrapDirection = DD_UP;
		}
		else {
			_iconWrapDirection = DD_DOWN;
		}
	}
	_iconWrapCount = max(_getInt("WrapCount", 10), 1);

	_getString("MouseLeftUp", _mouseLUp, ".none", MAX_PATH);
	_getString("MouseLeftDown", _mouseLDown, ".none", MAX_PATH);
	_getString("MouseLeftDbl", _mouseLDbl, ".open", MAX_PATH);
	_getString("MouseMiddleUp", _mouseMUp, ".none", MAX_PATH);
	_getString("MouseMiddleDown", _mouseMDown, ".none", MAX_PATH);
	_getString("MouseMiddleDbl", _mouseMDbl, ".none", MAX_PATH);
	_getString("MouseRightUp", _mouseRUp, ".context", MAX_PATH);
	_getString("MouseRightDown", _mouseRDown, ".none", MAX_PATH);
	_getString("MouseRightDbl", _mouseRDbl, ".none", MAX_PATH);

	// Icon effects
	_effectFlags = 0;
	_hueIntensity = (UCHAR)min(max(_getInt("HueIntensity", 0), 0), 255);
	_hueColor = _getColor("HueColor", RGB(128, 128, 128));
	if(_hueIntensity) {
		_effectFlags |= FX_HUE;
	}
	_saturation = (UCHAR)min(max(_getInt("Saturation", 255), 0), 255);
	if(_saturation != 255) {
		_effectFlags |= FX_SATURATION;
	}
}

Group::~Group()
{
	_deleteIcons();

	if(_iconTextFont) {
		DeleteObject(_iconTextFont);
	}
	if(_iconTextBackground) {
		DeleteObject(_iconTextBackground);
	}

	for(ShellNotifyListIter it = _notifiers.begin(); it != _notifiers.end(); it++) {
		delete (*it);
	}

	free(name);
}

void Group::show()
{
	_loadTooltips();
	_visible = TRUE;
	InvalidateRect(_parentWnd, NULL, TRUE);
}

void Group::hide()
{
	_deleteTooltips();
	_visible = FALSE;
	InvalidateRect(_parentWnd, NULL, TRUE);
}

void Group::toggle()
{
	if(_visible) {
		hide();
	}
	else {
		show();
	}
}

void Group::update()
{
	_deleteIcons();
	_loadIcons();
	InvalidateRect(_parentWnd, NULL, TRUE);
}

BOOL Group::mouseEvent(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if(!_visible) {
		return FALSE;
	}

	Icon* clicked = _getIconAtPt(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));

	if(clicked) {
		if(uMsg == WM_RBUTTONDOWN || uMsg == WM_LBUTTONDOWN || WM_MBUTTONDOWN) {
			if(_selectedIcon) {
				InvalidateRect(_parentWnd, &_selectedIcon->textRect, TRUE);
			}
			_selectedIcon = clicked;
			if(_selectedIcon) {
				InvalidateRect(_parentWnd, &_selectedIcon->textRect, TRUE);
			}
		}

		switch(uMsg) {
			case WM_RBUTTONUP:
				clicked->Execute(_mouseRUp);
				break;

			case WM_RBUTTONDOWN:
				clicked->Execute(_mouseRDown);
				break;

			case WM_RBUTTONDBLCLK:
				clicked->Execute(_mouseRDbl);
				break;

			case WM_LBUTTONUP: 
				clicked->Execute(_mouseLUp);
				break;

			case WM_LBUTTONDOWN:
				clicked->Execute(_mouseLDown);
				break;

			case WM_LBUTTONDBLCLK:
				clicked->Execute(_mouseLDbl);
				break;

			case WM_MBUTTONUP: 
				clicked->Execute(_mouseMUp);
				break;

			case WM_MBUTTONDOWN:
				clicked->Execute(_mouseMDown);
				break;

			case WM_MBUTTONDBLCLK:
				clicked->Execute(_mouseMDbl);
				break;
		}
		return TRUE;
	}
	else {
		if(_selectedIcon) {
			InvalidateRect(_parentWnd, &_selectedIcon->textRect, TRUE);
		}
		_selectedIcon = NULL;
		return FALSE;
	}
}

void Group::_deleteIcons()
{
	for(IconListIter it = _iconList.begin(); it != _iconList.end(); it++) {
		if((*it)->toolId) {
			DeleteTooltip((*it)->toolId);
		}

		delete (*it);
	}
	_iconList.clear();
	_iconsLoaded = FALSE;
}

void Group::_loadTooltips()
{
	if(_showTooltips) {
		for(IconListIter it = _iconList.begin(); it != _iconList.end(); it++) {
			(*it)->toolId = AddTooltip((*it)->title, (*it)->iconRect, (*it)->toolId);
		}
	}
}

void Group::_deleteTooltips()
{
	if(_showTooltips) {
		for(IconListIter it = _iconList.begin(); it != _iconList.end(); it++) {
			if((*it)->toolId) {
				DeleteTooltip((*it)->toolId);
			}
		}
	}
}

void Group::_loadFolder(const char* szFolder)
{
    LPMALLOC pMalloc;
    LPITEMIDLIST pidlFolder = NULL;
    LPITEMIDLIST pidlItems = NULL;
    IShellFolder *psfDesktop = NULL;
    IShellFolder *psfFolder = NULL;
    LPENUMIDLIST ppenum = NULL;
    ULONG celtFetched;
    HRESULT hr;
    STRRET strDispName;
    ULONG uAttr;
	WCHAR wszFolder[MAX_PATH];
	BOOL isDesktop = FALSE;
	char szMyDocuments[1024];

	// For memory managment
    hr = SHGetMalloc(&pMalloc);

	if(!stricmp(szFolder, ".desktop")) {
	    hr = SHGetDesktopFolder(&psfFolder);
		// Create a pidl with only the terminating NULLs
		pidlFolder = (LPITEMIDLIST)pMalloc->Alloc(2);
		ZeroMemory(pidlFolder, 2);
		isDesktop = TRUE;
		DWORD dwDataSize = 1024;
		SHGetValue(HKEY_CURRENT_USER, "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders", "Personal", NULL, szMyDocuments, &dwDataSize);
	}
	else {
		// Get the IShellFolder for the folder
		MultiByteToWideChar(CP_ACP, 0, szFolder, -1, wszFolder, sizeof(wszFolder) / sizeof(WCHAR));
		// Used to bind the interface
		hr = SHGetDesktopFolder(&psfDesktop);
		// Get the PIDL
		hr = psfDesktop->ParseDisplayName(NULL, NULL, wszFolder, NULL, &pidlFolder, NULL);
		// Get the interface
		hr = psfDesktop->BindToObject(pidlFolder, NULL, IID_IShellFolder, (LPVOID *)&psfFolder);
		// Release the desktop
		psfDesktop->Release();
	}

	// Get the enumeration object
	DWORD enumFlags = SHCONTF_FOLDERS | SHCONTF_NONFOLDERS;
	if(_iconShowHidden) {
		enumFlags |= SHCONTF_INCLUDEHIDDEN;
	}
    hr = psfFolder->EnumObjects(NULL, enumFlags, &ppenum);
    while(hr = ppenum->Next(1, &pidlItems, &celtFetched) == S_OK && (celtFetched) == 1) {
		// Create a icon object
		FileIcon* icon = new FileIcon();

		// Get the title
		uAttr = SFGAO_FILESYSTEM;
        psfFolder->GetAttributesOf(1, (LPCITEMIDLIST*)&pidlItems, &uAttr);
        psfFolder->GetDisplayNameOf(pidlItems, SHGDN_INFOLDER, &strDispName);
		StrRetToBuf(&strDispName, pidlItems, icon->title, MAX_PATH);

		// Get the filename
        psfFolder->GetDisplayNameOf(pidlItems, SHGDN_NORMAL | SHGDN_FORPARSING, &strDispName);
        StrRetToBuf(&strDispName, pidlItems, icon->filename, MAX_PATH);

		// Ugly hack to fix the My Documents icon
		if(isDesktop) {
			if(!strcmp(icon->filename, szMyDocuments)) {
				strcpy(icon->filename, "::{450D8FBA-AD25-11D0-98A8-0800361B1103}");
			}
		}

		// Get the full PIDL (relative to the desktop)
		icon->SetPIDL(AppendPIDL(pidlFolder, pidlItems, pMalloc));

		if(_iconHideExtension) {
			PathRemoveExtension(icon->title);
		}

		// Is it a folder?
		uAttr = SFGAO_FOLDER;
        psfFolder->GetAttributesOf(1, (LPCITEMIDLIST*)&pidlItems, &uAttr);
		if(uAttr & SFGAO_FOLDER) {
			icon->lFlags = icon->lFlags | Icon::IconFlags::ICON_FOLDER;
		}
		
		// Add it to the list
		_iconList.push_back((Icon*)icon);

		// Release the PIDL
        pMalloc->Free(pidlItems);
    }

	// Release interfaces/PIDLs
    ppenum->Release();
    pMalloc->Free(pidlFolder);
    pMalloc->Release();
    psfFolder->Release();
}

void Group::_loadIcons()
{
	for(StringListIter slit = _folders.begin(); slit != _folders.end(); slit++) {
		_loadFolder((*slit).c_str());
	}

	_iconList.sort(Group::_compareIcons);
	if(_iconFoldersFirst) {
		_iconList.sort(Group::_compareIconsFolder);
	}

	// Calculate RECT's and load icons
	int i = 0, x = 0, y = 0;
	IconListIter icon;
	for(icon = _iconList.begin(); icon != _iconList.end(); icon++) {
		switch(_iconDirection) {
			case DD_RIGHT:
			case DD_LEFT:
				x = (i % _iconWrapCount);
				y = (i / _iconWrapCount);
				if(_iconWrapDirection == DD_UP) {
					y = -y;
				}
				if(_iconDirection == DD_LEFT) {
					x = -x;
				}
				break;

			case DD_DOWN:
			case DD_UP:
				x = (i / _iconWrapCount);
				y = (i % _iconWrapCount);
				if(_iconWrapDirection == DD_LEFT) {
					x = -x;
				}
				if(_iconDirection == DD_UP) {
					y = -y;
				}
				break;

		}

		if(!_hideText) {
			(*icon)->x = _iconX + ((max(_iconSize, _iconTextWidth) + _iconSpacingX) * x);
			(*icon)->y = _iconY + ((_iconSize + _iconTextHeight + _iconTextSpacing + _iconSpacingY) * y);

			(*icon)->textRect.top = (*icon)->y + _iconSize + _iconTextSpacing;
			(*icon)->textRect.bottom = (*icon)->textRect.top + _iconTextHeight;
			(*icon)->textRect.left = (*icon)->x;// + (_iconSize / 2) - (_iconTextWidth / 2);
			(*icon)->textRect.right = (*icon)->textRect.left + _iconTextWidth;

			SetRect(&(*icon)->iconRect, (*icon)->x + (max(_iconSize, _iconTextWidth) / 2) - (_iconSize / 2), (*icon)->y, (*icon)->x + (max(_iconSize, _iconTextWidth) / 2) + (_iconSize / 2), (*icon)->y + _iconSize);
		}
		else {
			(*icon)->x = _iconX + ((_iconSize + _iconSpacingX) * x);
			(*icon)->y = _iconY + ((_iconSize + _iconSpacingY) * y);

			SetRect(&(*icon)->iconRect, (*icon)->x, (*icon)->y, (*icon)->x + _iconSize, (*icon)->y + _iconSize);
		}

		// Load icon
		(*icon)->SetIcon(_getIconFromPath((*icon)->filename, _iconSize, _iconSize));
		if(!(*icon)->GetIcon()) {
			LSLogPrintf(LOG_ERROR, "IconDesk", "Could not load icon for \"%s\"", (*icon)->filename);
		}

		// Create buffer copy with fx changes entered
		PAINTSTRUCT ps;
		HDC	hdcScreen = BeginPaint(_parentWnd, &ps);

		if(_effectFlags & FX_ANY) {
			if((*icon)->GetIcon()) {
				(*icon)->hdcBuffer = CreateCompatibleDC(hdcScreen);
				(*icon)->hbmBuffer = CreateCompatibleBitmap(hdcScreen, _iconSize, _iconSize);
				SelectObject((*icon)->hdcBuffer, (*icon)->hbmBuffer);

				int y = 0;
				HDC hdc = CreateCompatibleDC((*icon)->hdcBuffer);
				HBITMAP hbmNew = CreateCompatibleBitmap((*icon)->hdcBuffer, 2 * _iconSize, _iconSize);
				SelectObject(hdc, hbmNew);

				// Temporarily draw up the icon to read it's pixels
				DrawIconEx(hdc, 0, 0, (*icon)->GetIcon(), _iconSize, _iconSize, 0, NULL, DI_NORMAL);
				// the mask of the icon
				DrawIconEx(hdc, _iconSize, 0, (*icon)->GetIcon(), _iconSize, _iconSize, 0, NULL, DI_MASK);

				while(y < _iconSize) {
					int x = 0;
					while(x < _iconSize) {
						// Loop through all transparent pixels...
						while(x < _iconSize) {
							// If the mask-pixel is white, then break
							if(!GetPixel(hdc, x + _iconSize, y)) {
								COLORREF cl = GetPixel(hdc, x, y);
								BYTE r = GetRValue(cl);
								BYTE g = GetGValue(cl);
								BYTE b = GetBValue(cl);

								// Saturation effect
								if(_effectFlags & FX_SATURATION) {
									BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);

									r = (BYTE)((r * _saturation + gray * (255 - _saturation) + 255) >> 8);
									g = (BYTE)((g * _saturation + gray * (255 - _saturation) + 255) >> 8);
									b = (BYTE)((b * _saturation + gray * (255 - _saturation) + 255) >> 8);
								}
								// icon hue effect
								if(_effectFlags & FX_HUE) {
									// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
									// test normal
									r = (BYTE)((r * (255 - _hueIntensity) + GetRValue(_hueColor) * _hueIntensity + 255) >> 8);
									g = (BYTE)((g * (255 - _hueIntensity) + GetGValue(_hueColor) * _hueIntensity + 255) >> 8);
									b = (BYTE)((b * (255 - _hueIntensity) + GetBValue(_hueColor) * _hueIntensity + 255) >> 8);
								}
								SetPixel((*icon)->hdcBuffer, x, y, RGB(r, g, b));
							}
							// for transparent blt?
							else {
								SetPixel((*icon)->hdcBuffer, x, y, RGB(255, 0, 255));
							}
							x++;
						}
					}
					y++;
				}
				DeleteObject(hbmNew);
				DeleteDC(hdc);
			}
		}
		DeleteObject(hdcScreen);
		EndPaint(_parentWnd, &ps);

		i++;
	}

	_loadTooltips();

	_iconsLoaded = TRUE;
}

bool Group::_compareIconsFolder(Icon* i1, Icon* i2)
{
	return ((i1->lFlags & Icon::IconFlags::ICON_FOLDER) > (i2->lFlags & Icon::IconFlags::ICON_FOLDER));
};

bool Group::_compareIcons(Icon* i1, Icon* i2)
{
	return (stricmp(i1->title, i2->title) < 0);
};

void Group::paintIcons(HDC hDC)
{
	if(_visible) {
		if(_iconsLoaded == FALSE) {
			_loadIcons();
		}

		COLORREF oldColor = SetTextColor(hDC, _iconTextColor);
		HFONT oldFont = (HFONT)SelectObject(hDC, _iconTextFont);
		int oldMode = SetBkMode(hDC, TRANSPARENT);

		for(IconListIter it = _iconList.begin(); it != _iconList.end(); it++) {
			if((*it)->GetIcon()) {
				if(_effectFlags & FX_ANY) {
					// Paint from the buffer
					TransparentBltLS(hDC, (*it)->x + (max(_iconSize, _iconTextWidth) / 2) - (_iconSize / 2), (*it)->y, _iconSize, _iconSize, (*it)->hdcBuffer, 0, 0, RGB(255, 0, 255));
				}
				else {
					DrawIconEx(hDC, (*it)->x + (max(_iconSize, _iconTextWidth) / 2) - (_iconSize / 2), (*it)->y, (*it)->GetIcon(), _iconSize, _iconSize, 0, NULL, DI_NORMAL);
				}
			}
			if(!_hideText) {
				if(_iconTextBackground) {
					FillRect(hDC, &(*it)->textRect, _iconTextBackground);
				}
				if(_selectedIcon == (*it)) {
					DrawFocusRect(hDC, &(*it)->textRect);
				}
				if(_textShadowDepth) {
					RECT r;
					CopyRect(&r, &(*it)->textRect);
					OffsetRect(&r, _textShadowDepth, _textShadowDepth);
					SetTextColor(hDC, _textShadowColor);
					DrawText(hDC, (*it)->title, strlen((*it)->title), &r, DT_WORDBREAK | DT_CENTER | DT_NOPREFIX /*| DT_NOCLIP*/ | DT_END_ELLIPSIS);
					SetTextColor(hDC, _iconTextColor);
				}
				DrawText(hDC, (*it)->title, strlen((*it)->title), &(*it)->textRect, DT_WORDBREAK | DT_CENTER | DT_NOPREFIX /*| DT_NOCLIP*/ | DT_END_ELLIPSIS);
			}
		}

		SetTextColor(hDC, oldColor);
		SelectObject(hDC, oldFont);
		SetBkMode(hDC, oldMode);
	}
}

Icon* Group::_getIconAtPt(int x, int y)
{
	for(IconListIter it = _iconList.begin(); it != _iconList.end(); it++) {
		if((*it)->PointOver(x, y)) {
			return (*it);
		}
	}
	return NULL;
}

HICON Group::_getIconFromPath(LPCSTR pszPath, int cxDesired, int cyDesired)
{
	if(!PrivateExtractIcons) {
		return NULL;
	}

	HICON hIcon = NULL;
	LPCSTR pszExt;
    pszExt = strrchr(pszPath, '.');

	LSLogPrintf(LOG_DEBUG, "IconDesk", "Loading icon for '%s'", pszPath);
        
    if(pszExt &&  (!stricmp(pszExt, ".ico") || !stricmp(pszExt, ".cur") || !stricmp(pszExt, ".ani") || !stricmp(pszExt, ".exe"))) {
		// Executables, icons and cursors have their own icons
		PrivateExtractIcons(pszPath, 0, cxDesired, cyDesired, &hIcon, NULL, 1, NULL);
    }
    else if(pszExt && !stricmp(pszExt, ".lnk")) {
		// Special case for link files
		// If the icon is set for the link, retrieve it
		SHFILEINFO shfi;
		SHGetFileInfo(pszPath, 0, &shfi, sizeof(SHFILEINFO), SHGFI_ICONLOCATION);
		// Extract the icon
		if(PrivateExtractIcons(shfi.szDisplayName, shfi.iIcon, cxDesired, cyDesired, &hIcon, NULL, 1, NULL) == 0xFFFFFFFF) {
			// If the icon was not explicitly set in the linkfile, we'll use the icon from the target file
			HRESULT hRes;				
			char szInfo[1024];
			IShellLink* psl;

			// Create the ShellLink object
			hRes = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, IID_IShellLink, (LPVOID*)&psl);
			if(SUCCEEDED(hRes)) {
				IPersistFile* ppf;
				// Bind the ShellLink object to the Persistent File
				hRes = psl->QueryInterface(IID_IPersistFile, (LPVOID *) &ppf);
				if(SUCCEEDED(hRes)) {
					WORD wsz[MAX_PATH];

					MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, pszPath, -1, wsz, MAX_PATH);
					hRes = ppf->Load(wsz, 0);
					
					if(SUCCEEDED(hRes)) {
						// Retrieve the target of the link file
						psl->GetPath(szInfo, 1024, NULL, SLGP_UNCPRIORITY);
					}
				}
			}
			psl->Release();

			// Extract the icon from the link target
			PrivateExtractIcons(szInfo, 0, cxDesired, cyDesired, &hIcon, NULL, 1, NULL);
		}
    }
    else {
		// Load icons for all other file types
		if(pszExt) {
			// Retrieve the icon associated with the file type from the registry
			char szFileType[1024], szIcon[1024];
			int iIconIndex;
			DWORD dwDataSize = 1024;
			// Get the file type from the extension
			if(SHGetValue(HKEY_CLASSES_ROOT, pszExt, NULL, NULL, szFileType, &dwDataSize) == ERROR_SUCCESS) {
				strcat(szFileType, "\\DefaultIcon");
				dwDataSize = 1024;
				// Get the icon
				if(SHGetValue(HKEY_CLASSES_ROOT, szFileType, NULL, NULL, szIcon, &dwDataSize) == ERROR_SUCCESS) {
					iIconIndex = atoi(strrchr(szIcon, ',') + 1);
					(*strrchr(szIcon, ',')) = '\0';

					// Extract the icon
					PrivateExtractIcons(szIcon, iIconIndex, cxDesired, cyDesired, &hIcon, NULL, 1, NULL);
				}
			}

			if(!hIcon) {
				// No icon
				PrivateExtractIcons("shell32.dll", 0, cxDesired, cyDesired, &hIcon, NULL, 1, NULL);
			}
		}
		else {
			// Let the shell decide the icon
			SHFILEINFO shfi;
			SHGetFileInfo(pszPath, 0, &shfi, sizeof(SHFILEINFO), SHGFI_ICONLOCATION | SHGFI_SELECTED);
			// Extract the icon
			PrivateExtractIcons(shfi.szDisplayName, shfi.iIcon, cxDesired, cyDesired, &hIcon, NULL, 1, NULL);
		}
    }
	return hIcon;
}

void Group::_getString(const char* key, char* value, const char* defaultValue, int maxLen)
{
	char buffer[MAX_RCCOMMAND] = "";
	sprintf(buffer, "IconDesk%s%s", name, key);
	GetRCString(buffer, value, defaultValue, maxLen);
}

BOOL Group::_getBool(const char* key, BOOL defaultValue)
{
	char buffer[MAX_RCCOMMAND] = "";
	sprintf(buffer, "IconDesk%s%s", name, key);
	return GetRCBoolDef(buffer, defaultValue);
}

int Group::_getInt(const char* key, int defaultValue)
{
	char buffer[MAX_RCCOMMAND] = "";
	sprintf(buffer, "IconDesk%s%s", name, key);
	return GetRCInt(buffer, defaultValue);
}

COLORREF Group::_getColor(const char* key, int defaultValue)
{
	char buffer[MAX_RCCOMMAND] = "";
	sprintf(buffer, "IconDesk%s%s", name, key);
	return GetRCColor(buffer, defaultValue);
}
