/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef __GROUP_H
#define __GROUP_H

#include <windows.h>
#include <windowsx.h>
#include <shlwapi.h>
#include <commctrl.h>
#include <shlobj.h>
#include <list>
#include <string>
#include "..\lsapi\lsapi.h"
#include "..\lsapi\lswinbase.h"
#include "pidl.h"
#include "tooltips.h"
#include "Icon.h"
#include "FileIcon.h"
#include "ShellNotify.h"

using namespace std;

enum DESKTOPDIRECTION
{
	DD_RIGHT,
	DD_DOWN,
	DD_LEFT,
	DD_UP
};

// Defined effects
#define FX_HUE			0x1
#define FX_SATURATION	0x2
#define FX_ANY			(FX_HUE | FX_SATURATION)

class ShellNotify;

class Group
{
private:
	typedef list<Icon*> IconList;
	typedef IconList::iterator IconListIter;

	typedef list<string> StringList;
	typedef StringList::iterator StringListIter;

	typedef list<ShellNotify*> ShellNotifyList;
	typedef ShellNotifyList::iterator ShellNotifyListIter;

public:
	Group(const char *name, HWND parentWnd);
	~Group();

	void show();
	void hide();
	void toggle();

	void paintIcons(HDC hDC);
	void update();
	BOOL mouseEvent(UINT uMsg, WPARAM wParam, LPARAM lParam);
	
	char* name;

private:
	void _loadIcons();
	void _loadFolder(const char* szFolder);
	void _deleteIcons();

	void _loadTooltips();
	void _deleteTooltips();

	Icon* _getIconAtPt(int x, int y);
	Icon* _selectedIcon;

	HICON _getIconFromPath(LPCSTR pszPath, int cxDesired, int cyDesired);

	void _getString(const char* key, char* value, const char* defaultValue, int maxLen);
	BOOL _getBool(const char* key, BOOL defaultValue);
	int _getInt(const char* key, int defaultValue);
	COLORREF _getColor(const char* key, int defaultValue);

	static bool _compareIcons(Icon* i1, Icon* i2);
	static bool _compareIconsFolder(Icon* i1, Icon* i2);

	HWND _parentWnd;
	BOOL _visible;

	IconList _iconList;
	BOOL _iconsLoaded;

	int _iconX;
	int _iconY;
	int _iconSize;
	int _iconSpacingX;
	int _iconSpacingY;

	StringList _folders;
	ShellNotifyList _notifiers;

	char _mouseLUp[MAX_PATH];
	char _mouseMUp[MAX_PATH];
	char _mouseRUp[MAX_PATH];
	char _mouseLDown[MAX_PATH];
	char _mouseMDown[MAX_PATH];
	char _mouseRDown[MAX_PATH];
	char _mouseLDbl[MAX_PATH];
	char _mouseMDbl[MAX_PATH];
	char _mouseRDbl[MAX_PATH];

	BOOL _iconFoldersFirst;
	BOOL _iconHideExtension;
	BOOL _iconShowHidden;
	BOOL _hideText;
	BOOL _showTooltips;

	DESKTOPDIRECTION _iconDirection;
	DESKTOPDIRECTION _iconWrapDirection;
	int _iconWrapCount;

	int _iconTextSpacing;
	int _iconTextHeight;
	int _iconTextWidth;
	HFONT _iconTextFont;
	COLORREF _iconTextColor;
	HBRUSH _iconTextBackground;

	COLORREF _textShadowColor;
	int _textShadowDepth;

	// Icon effects
	COLORREF _hueColor;
	UCHAR _hueIntensity;
	UCHAR _saturation;
	UCHAR _effectFlags;

	UINT (WINAPI *PrivateExtractIcons)(LPCTSTR,int,int,int,HICON*,UINT*,UINT,UINT);

};

#endif//__GROUP_H
